--
-- Bizon Z020
-- author: Burner
--

zagon = {};

function zagon.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function zagon:load(xmlFile)
	self.currentFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
	self.wasToFast = false;

	local numStrawAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.strawAreas#count"), 0);
	self.strawAreas = {}
	for i=1, numStrawAreas do
		local area = {};
		local areanamei = string.format("vehicle.strawAreas.strawArea%d", i);
		area.start = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#startIndex"));
		area.width = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#widthIndex"));
		area.height = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#heightIndex"));
		table.insert(self.strawAreas, area);
	end;

	local fakeWheel = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.fakeWheel#index"));
	if fakeWheel ~= nil then
		self.fakeWheel = fakeWheel;
	end;
	
	local fakeWheelPosition = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.fakeWheelPosition#index"));
	if fakeWheelPosition ~= nil then
		self.fakeWheelPosition = fakeWheelPosition;
	end;
	
	local cutterShape = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.cutter#index"));
	if cutterShape ~= nil then
		self.cutterShape = cutterShape;
	end;
	
	local cutterReel = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.cutterReel#index"));
	if cutterReel ~= nil then
		self.cutterReel = cutterReel;
	end;
	
	local cutterRoll = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.cutterRoll#index"));
	if cutterRoll ~= nil then
		self.cutterRoll = cutterRoll;
	end;

        local pRoll = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.pRoll#index"));
	if pRoll ~= nil then
		self.pRoll = pRoll;
	end;
	
    --local indexSpikesStr = getXMLString(xmlFile, "vehicle.cutterReelSpikes#index");
    --self.spikesCount = getXMLInt(xmlFile, "vehicle.cutterReelSpikes#count");
    --self.spikesRootNode = Utils.indexToObject(self.components, indexSpikesStr);
	
	local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.cutter#detachedPosition"));
    self.cutterDetachedPosition = {};
    self.cutterDetachedPosition.x = Utils.getNoNil(x, 0);
    self.cutterDetachedPosition.z = Utils.getNoNil(z, 0);
	
	local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.cutter#attachedPosition"));
    self.cutterAttachedPosition = {};
    self.cutterAttachedPosition.x = Utils.getNoNil(x, 0);
	self.cutterAttachedPosition.y = Utils.getNoNil(y, 0);
    self.cutterAttachedPosition.z = Utils.getNoNil(z, 0);

	local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.cutter#detachedRotation"));
    self.cutterDetachedRotation = {};
    self.cutterDetachedRotation.x = Utils.degToRad(Utils.getNoNil(x, 0));
    self.cutterDetachedRotation.y = Utils.degToRad(Utils.getNoNil(y, 0));
    self.cutterDetachedRotation.z = Utils.degToRad(Utils.getNoNil(z, 0));

	local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.cutter#attachedRotation"));
    self.cutterAttachedRotation = {};
    self.cutterAttachedRotation.x = Utils.degToRad(Utils.getNoNil(x, 0));
    self.cutterAttachedRotation.y = Utils.degToRad(Utils.getNoNil(y, 0));
    self.cutterAttachedRotation.z = Utils.degToRad(Utils.getNoNil(z, 0));

	local cutterBaseNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.cutterBase#index"));
    if cutterBaseNode ~= nil then
        self.cutterBase = {};
        self.cutterBase.node = cutterBaseNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.cutterBase#minRot"));
        self.cutterBase.minRot = {};
        self.cutterBase.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.cutterBase.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.cutterBase.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.cutterBase#maxRot"));
        self.cutterBase.maxRot = {};
        self.cutterBase.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.cutterBase.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.cutterBase.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.cutterBase.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.cutterBase#rotTime"), 2)*1000;
        self.cutterBase.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.cutterBase#touchRotLimit"), 10));
    end;
	
	self.cutterLift = false;
	self.threshingActivated = false;
	
	self.strawToggleTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.strawParticleSystems#toggleTime"), 2500);
	self.strawEnableTime = nil;
	self.strawDisableTime = nil;
	self.strawEmitState = false;
	
	self.chopperToggleTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.chopperParticleSystems#toggleTime"), 2500);
	self.chopperEnableTime = nil;
	self.chopperDisableTime = nil;
	self.chopperEmitState = false;
	
	self.strawParticle = {};
    local i = 0;
    while true do
        local namei = string.format("vehicle.strawParticleSystems.strawParticleSystem(%d)", i);
		local nodei = Utils.indexToObject(self.components, getXMLString(xmlFile, namei .. "#index"));
		if nodei == nil then
			break;
		end; 
        Utils.loadParticleSystem(xmlFile, self.strawParticle, namei, nodei, false, nil, self.baseDirectory)		
		Utils.setEmittingState(self.strawParticle,false);
		i = i +1;		
    end;
	
	self.chopperParticle = {};
    local i = 0;
    while true do
        local namei = string.format("vehicle.chopperParticleSystems.chopperParticleSystem(%d)", i);
		local nodei = Utils.indexToObject(self.components, getXMLString(xmlFile, namei .. "#index"));
		if nodei == nil then
			break;
		end; 
        Utils.loadParticleSystem(xmlFile, self.chopperParticle, namei, nodei, false, nil, self.baseDirectory)		
		Utils.setEmittingState(self.chopperParticle,false);
		i = i +1;		
    end;
	
	self.cutterParticle = {};
    local i = 0;
    while true do
        local namei = string.format("vehicle.cutterParticleSystems.cutterParticleSystem(%d)", i);
		local nodei = Utils.indexToObject(self.components, getXMLString(xmlFile, namei .. "#index"));
		if nodei == nil then
			break;
		end; 
        Utils.loadParticleSystem(xmlFile, self.cutterParticle, namei, nodei, false, nil, self.baseDirectory)		
		Utils.setEmittingState(self.cutterParticle,false);
		i = i +1;		
    end;
	
	local workSound = getXMLString(xmlFile, "vehicle.workSound#file");
	self.soundTest = workSound;
    if workSound ~= nil and workSound ~= "" then
        workSound = Utils.getFilename(workSound, self.baseDirectory);
        self.workSound = createSample("workSound");
        self.workSoundEnabled = false;
        loadSample(self.workSound, workSound, false);
        self.workSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.workSound#pitchOffset"), 1);
        self.workSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.workSound#volume"), 1);
    end;
end;

function zagon:update(dt)
	local disableStrawEmit = true;
	local disableChopperEmit = true;

	if self.cutterBase ~= nil then
		local x, y, z = getRotation(self.cutterBase.node);
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.cutterBase.maxRot, self.cutterBase.minRot, 3, self.cutterBase.rotTime, dt, not self.cutterLift);
		setRotation(self.cutterBase.node,unpack(newRot));	
	end;
			
    if self.attacherVehicle then
		if self:getIsActive() then
			if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) then
				self.cutterLift = not self.cutterLift;
			end;
			
			if InputBinding.hasEvent(InputBinding.ACTIVATE_THRESHING) then
				self.threshingActivated = not self.threshingActivated;
			end;

			if self.threshingActivated and self.fillLevel < self.capacity then
				if not self.workSoundEnabled and self:getIsActiveForSound() then
					playSample(self.workSound, 0, self.workSoundVolume, 0);
					setSamplePitch(self.workSound, self.workSoundPitchOffset);
					self.workSoundEnabled = true;
				end;
				
				rotate(self.cutterReel, dt*0.0045, 0, 0);
				rotate(self.cutterRoll, dt*0.003*3, 0, 0);
                                rotate(self.pRoll, dt*0.003*3, 0, 0);
				
				local atx, aty, atz = getRotation(self.cutterReel);
	 
				--for i=1, self.spikesCount do
				--	local spike = getChildAt(self.spikesRootNode, i-1);
				--	setRotation(spike, -atx, aty, atz);
				--end;

				local toFast = self:doCheckSpeedLimit() and self.attacherVehicle.lastSpeed*3600 > 20;
				if not toFast and not self.cutterLift then
					local area =0;
					for k, cuttingArea in pairs(self.cuttingAreas) do
						local x,y,z = getWorldTranslation(cuttingArea.start);
						local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
						local x2,y2,z2 = getWorldTranslation(cuttingArea.height);
						for fillType,v in pairs(self.fillTypes) do
							if fillType ~= Fillable.FILLTYPE_UNKNOWN then
								if self.currentFillType == fillType or self.currentFillType == Fillable.FILLTYPE_UNKNOWN then
									local fruitType = FruitUtil.fillTypeToFruitType[fillType];
									Utils.updateFruitCutShortArea(fruitType, x, z, x1, z1, x2, z2, 1);
									local area = Utils.cutFruitArea(fruitType, x, z, x1, z1, x2, z2, 0)
									area = area + Utils.cutFruitArea(fruitType, x, z, x1, z1, x2, z2, 0);
									if area > 0 then
										self.currentFruitType = fruitType;
										Utils.setEmittingState(self.cutterParticle,true);
										local fruitDesc = FruitUtil.fruitIndexToDesc[fruitType];
										if fruitDesc.hasStraw then
											if self.strawEnableTime == nil then
												self.strawEnableTime = self.time + self.strawToggleTime;
											else
												self.strawDisableTime = nil;
											end;
											self.strawPSFruitType = fruitType;
											disableStrawEmit = false;
										else
											if self.chopperEnableTime == nil then
												self.chopperEnableTime = self.time + self.chopperToggleTime;
											else
												self.chopperDisableTime = nil;
											end;
											disableChopperEmit = false;										
										end;
										local pixelToQm = 2048 / 4096 * 2048 / 4096  / g_currentMission.maxFruitValue; -- 4096px are mapped to 2048m
										local literPerQm = FruitUtil.fruitIndexToDesc[fruitType].literPerSqm * (0.50 * (3 - g_currentMission.missionStats.difficulty));
										local qm = area*pixelToQm;
										local deltaLevel = qm*literPerQm * self.fillScale;
										self:setFillLevel(self.fillLevel+deltaLevel, fillType);
									else
										Utils.setEmittingState(self.cutterParticle,false);
									end;
								end;
							end;
						end;
					end;							
				end;
				self.wasToFast = toFast;
			else
				self.threshingActivated = false;
				if self.workSoundEnabled then
					stopSample(self.workSound);
					self.workSoundEnabled = false;
				end;
				Utils.setEmittingState(self.cutterParticle,false);
			end;
			
			if disableStrawEmit and self.strawDisableTime == nil then
				self.strawDisableTime = self.time + self.strawToggleTime;
			end;
			
			if self.strawEnableTime ~= nil and self.strawEnableTime <= self.time then
				self.strawPSenabled = true;
				self.strawEnableTime = nil;
				self.strawEmitState = true;
			end;
			
			if self.strawEmitState then
				Utils.setEmittingState(self.strawParticle, true);
				for k, strawArea in pairs(self.strawAreas) do
					local x,y,z = getWorldTranslation(strawArea.start);
					local x1,y1,z1 = getWorldTranslation(strawArea.width);
					local x2,y2,z2 = getWorldTranslation(strawArea.height);
					local old, total = Utils.getFruitWindrowArea(self.strawPSFruitType, x, z, x1, z1, x2, z2);
					local value = 1+math.floor(old / total + 0.7); -- round, biased to the bigger value
					value = math.min(value, g_currentMission.maxWindrowValue);
					Utils.updateFruitWindrowArea(self.strawPSFruitType, x, z, x1, z1, x2, z2, value, true);
				end;
			end;
			
			if disableChopperEmit and self.chopperDisableTime == nil then
				self.chopperDisableTime = self.time + self.chopperToggleTime;
			end;
			
			if self.chopperEnableTime ~= nil and self.chopperEnableTime <= self.time then
				self.chopperPSenabled = true;
				self.chopperEnableTime = nil;
				self.chopperEmitState = true;
			end;
			
			if self.chopperEmitState then
				Utils.setEmittingState(self.chopperParticle, true);
			end;

			for k,wheel in pairs(self.attacherVehicle.wheels) do
				if wheel.driveMode == 2 then
					wrx,wry,wrz = getRotation(wheel.driveNode);
					break;
				end;
			end;
			setRotation(self.fakeWheel,wrx,wry,wrz);
		end;
		
		setTranslation(self.cutterShape,self.cutterAttachedPosition.x,self.cutterAttachedPosition.y,self.cutterAttachedPosition.z)		
		setRotation(self.cutterShape,self.cutterAttachedRotation.x,self.cutterAttachedRotation.y,self.cutterAttachedRotation.z);

	else
		local x,y,z = getWorldTranslation(self.cutterShape);
		local tH = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
		setRotation(self.cutterShape,self.cutterDetachedRotation.x,self.cutterDetachedRotation.y,self.cutterDetachedRotation.z);
		local relHeight = tH + 1.3;
		Utils.setWorldTranslation(self.cutterShape,x,relHeight,z)
		local x,y,z = getTranslation(self.cutterShape)
		setTranslation(self.cutterShape,self.cutterDetachedPosition.x,y,self.cutterDetachedPosition.z)	
	end;

	local x,y,z = getWorldTranslation(self.fakeWheel)
	local x11,y11,z11 = getWorldTranslation(self.fakeWheelPosition)
	local x1,y1,z1 = getTranslation(self.fakeWheel)
	local tH = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
	local relHeight = y - tH;
	local wheeldiff = y11 - tH;
	if wheeldiff < 1 then
		local relHeight = tH + 0.54;
		Utils.setWorldTranslation(self.fakeWheel,x,relHeight,z)
	end;	

	if self:getIsActive() then
		if self.strawDisableTime ~= nil and self.strawDisableTime <= self.time then
			Utils.setEmittingState(self.strawParticle, false);
			self.strawPSenabled = false;
			self.strawDisableTime = nil;
			self.strawEmitState = false;
		end;
		
		if self.chopperDisableTime ~= nil and self.chopperDisableTime <= self.time then
			Utils.setEmittingState(self.chopperParticle, false);
			self.chopperPSenabled = false;
			self.chopperDisableTime = nil;
			self.chopperEmitState = false;
		end;
	end;
	
	if self.fillLevel == 0 then
		self.currentFillType = Fillable.FILLTYPE_UNKNOWN;
	end;
end;

function zagon:mouseEvent(posX, posY, isDown, isUp, button)
end;

function zagon:keyEvent(unicode, sym, modifier, isDown)
end;

function zagon:draw()
	if self.threshingActivated then
		g_currentMission:addHelpButtonText("De-Activate Threshing", InputBinding.ACTIVATE_THRESHING);	
    else
		g_currentMission:addHelpButtonText("Activate Threshing", InputBinding.ACTIVATE_THRESHING);	
    end;
	if self.cutterLift then
		g_currentMission:addHelpButtonText("Lower Header", InputBinding.LOWER_IMPLEMENT);	
    else
		g_currentMission:addHelpButtonText("Raise Header", InputBinding.LOWER_IMPLEMENT);	
    end;
	if self.wasToFast then
		g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "1", InputBinding.getKeyNamesOfDigitalAction(InputBinding.SPEED_LEVEL1)), 0.07+0.022, 0.019+0.029);
	end;
end;

function zagon:onAttach()
	self.cutterLift = true;
end; 	

function zagon:onDetach()
	self.wasToFast = false;
	self.strawEnableTime = nil;
	self.strawDisableTime = nil;
	self.strawEmitState = false;
	self.chopperEnableTime = nil;
	self.chopperDisableTime = nil;
	self.chopperEmitState = false;
	self.threshingActivated = false;
	setRotation(self.cutterBase.node,0,0,0);
	self.cutterLift = false;
	Utils.setEmittingState(self.cutterParticle,false);
	Utils.setEmittingState(self.strawParticle, false);
	Utils.setEmittingState(self.chopperParticle, false);
	if self.workSoundEnabled then
		stopSample(self.workSound);
		self.workSoundEnabled = false;
	end;
end;

function zagon:onDeactivate()
	self.wasToFast = false;
	self.threshingActivated = false;
	self.strawEnableTime = nil;
	self.strawDisableTime = nil;
	self.strawEmitState = false;
	self.chopperEnableTime = nil;
	self.chopperDisableTime = nil;
	self.chopperEmitState = false;
	Utils.setEmittingState(self.cutterParticle,false);
	Utils.setEmittingState(self.strawParticle, false);
	Utils.setEmittingState(self.chopperParticle, false);
	if self.workSoundEnabled then
		stopSample(self.workSound);
		self.workSoundEnabled = false;
	end;
end;

function zagon:delete()
	Utils.deleteParticleSystem(self.strawParticle);
	Utils.deleteParticleSystem(self.chopperParticle);
	Utils.deleteParticleSystem(self.cutterParticle);
	self.workSoundEnabled = false;
	if self.workSound ~= nil then
		delete(self.workSound);
	end;
end;

function zagon:onLeave()
	self.threshingActivated = false;
	self.strawEnableTime = nil;
	self.strawDisableTime = nil;
	self.strawEmitState = false;
	self.chopperEnableTime = nil;
	self.chopperDisableTime = nil;
	self.chopperEmitState = false;
	Utils.setEmittingState(self.cutterParticle,false);
	Utils.setEmittingState(self.strawParticle, false);
	Utils.setEmittingState(self.chopperParticle, false);
	if self.workSoundEnabled then
		stopSample(self.workSound);
		self.workSoundEnabled = false;
	end;
end;